@@ -286,7 +286,7 @@ nautilus_menu_item_get_type (void)
 	static GType type = 0;
 	
 	if (!type) {
-		static const GTypeInfo info = {
+		const GTypeInfo info = {
 			sizeof (NautilusMenuItemClass),
 			NULL,
 			NULL,
