@@ -35,7 +35,7 @@
 #include <libnautilus-private/nautilus-directory.h>
 #include <libnautilus-private/nautilus-file-attributes.h>
 #include <libnautilus-private/nautilus-file.h>
-#include <gtk/gtkenums.h>
+#include <gtk/gtk.h>
 #include <string.h>
 
 enum {
@@ -667,12 +667,13 @@ report_node_inserted (FMTreeModel *model, TreeNode *node)
 	report_row_inserted (model, &iter);
 	node->inserted = TRUE;
 
+	if (tree_node_has_dummy_child (node)) {
+		report_dummy_row_inserted (model, node);
+	}
+
 	if (node->directory != NULL ||
 	    node->parent == NULL) {
 		report_row_has_child_toggled (model, &iter);
-       }
-       if (tree_node_has_dummy_child (node)) {
-               report_dummy_row_inserted (model, node);
 	}
 }
 
@@ -1704,6 +1705,7 @@ fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 		return;
 	}
 	model->details->show_hidden_files = show_hidden_files;
+	model->details->show_backup_files = show_hidden_files;
 	stop_monitoring (model);
 	if (!show_hidden_files) {
 		destroy_by_function (model, nautilus_file_is_hidden_file);
@@ -1711,25 +1713,6 @@ fm_tree_model_set_show_hidden_files (FMTreeModel *model,
 	schedule_monitoring_update (model);
 }
 
-void
-fm_tree_model_set_show_backup_files (FMTreeModel *model,
-					   gboolean show_backup_files)
-{
-	g_return_if_fail (FM_IS_TREE_MODEL (model));
-	g_return_if_fail (show_backup_files == FALSE || show_backup_files == TRUE);
-
-	show_backup_files = show_backup_files != FALSE;
-	if (model->details->show_backup_files == show_backup_files) {
-		return;
-	}
-	model->details->show_backup_files = show_backup_files;
-	stop_monitoring (model);
-	if (!show_backup_files) {
-		destroy_by_function (model, nautilus_file_is_backup_file);
-	}
-	schedule_monitoring_update (model);
-}
-
 static gboolean
 file_is_not_directory (NautilusFile *file)
 {
@@ -1767,6 +1750,40 @@ fm_tree_model_iter_get_file (FMTreeModel *model, GtkTreeIter *iter)
 	return node == NULL ? NULL : nautilus_file_ref (node->file);
 }
 
+/* This is used to work around some sort order stability problems
+   with gtktreemodelsort */
+int
+fm_tree_model_iter_compare_roots (FMTreeModel *model,
+				  GtkTreeIter *iter_a,
+				  GtkTreeIter *iter_b)
+{
+	TreeNode *a, *b, *n;
+
+	g_return_val_if_fail (FM_IS_TREE_MODEL (model), 0);
+	g_return_val_if_fail (iter_is_valid (model, iter_a), 0);
+	g_return_val_if_fail (iter_is_valid (model, iter_b), 0);
+	
+	a = iter_a->user_data;
+	b = iter_b->user_data;
+	
+	g_assert (a != NULL && a->parent == NULL);
+	g_assert (b != NULL && b->parent == NULL);
+
+	if (a == b) {
+		return 0;
+	}
+	
+	for (n = model->details->root_node; n != NULL; n = n->next) {
+		if (n == a) {
+			return -1;
+		}
+		if (n == b) {
+			return 1;
+		}
+	}
+	g_assert_not_reached ();
+}
+
 gboolean
 fm_tree_model_iter_is_root (FMTreeModel *model, GtkTreeIter *iter)
 {
