@@ -59,6 +59,8 @@
 #include <libnautilus-private/nautilus-cell-renderer-pixbuf-emblem.h>
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
+#include <libnautilus-private/nautilus-window-info.h>
+#include <libnautilus-private/nautilus-window-slot-info.h>
 
 typedef struct {
         GObject parent;
@@ -78,7 +80,7 @@ struct FMTreeViewDetails {
 	GVolumeMonitor *volume_monitor;
 
 	NautilusFile *activation_file;
-	gboolean activation_in_new_window;
+	NautilusWindowOpenFlags activation_flags;
 
 	NautilusTreeViewDragDest *drag_dest;
 
@@ -100,6 +102,7 @@ struct FMTreeViewDetails {
 	GtkWidget *popup_properties;
 	GtkWidget *popup_unmount_separator;
 	GtkWidget *popup_unmount;
+	GtkWidget *popup_eject;
 	NautilusFile *popup_file;
 	
 	guint selection_changed_timer;
@@ -113,16 +116,11 @@ typedef struct {
 static GdkAtom copied_files_atom;
 static gboolean show_delete_command_auto_value;
 
-enum {
-	GNOME_COPIED_FILES
-};
-
-static const GtkTargetEntry clipboard_targets[] = {
-	{ "x-special/gnome-copied-files", 0, GNOME_COPIED_FILES },
-};
-
 static void  fm_tree_view_iface_init        (NautilusSidebarIface         *iface);
 static void  sidebar_provider_iface_init    (NautilusSidebarProviderIface *iface);
+static void  fm_tree_view_activate_file     (FMTreeView *view, 
+			    		     NautilusFile *file,
+					     NautilusWindowOpenFlags flags);
 static GType fm_tree_view_provider_get_type (void);
 
 static void create_popup_menu (FMTreeView *view);
@@ -322,16 +320,22 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
         char *uri, *file_uri;
         FMTreeView *view;
 	GdkScreen *screen;
-	NautilusWindowOpenMode mode;
 	GFile *location;
+	NautilusWindowSlotInfo *slot;
+	gboolean open_in_same_slot;
 	
         view = FM_TREE_VIEW (callback_data);
 
 	screen = gtk_widget_get_screen (GTK_WIDGET (view->details->tree_widget));
 
         g_assert (file == view->details->activation_file);
-        
-        mode = view->details->activation_in_new_window ? NAUTILUS_WINDOW_OPEN_IN_NAVIGATION : NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE;
+
+	open_in_same_slot =
+		(view->details->activation_flags &
+		 (NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW |
+		  NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB)) == 0;
+
+	slot = nautilus_window_info_get_active_slot (view->details->window);
 
 	uri = nautilus_file_get_activation_uri (file);
 	if (nautilus_file_is_launcher (file)) {
@@ -354,37 +358,41 @@ got_activation_uri_callback (NautilusFile *file, gpointer callback_data)
 					    "tree view window_info_open_location window=%p: %s",
 					    view->details->window, uri);
 			location = g_file_new_for_uri (uri);
-			nautilus_window_info_open_location
-				(view->details->window, 
+			nautilus_window_slot_info_open_location
+				(slot,
 				 location, 
-				 mode,
-				 0,
+				 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+				 view->details->activation_flags,
 				 NULL);
 			g_object_unref (location);
 		} else {
 			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 					    "tree view launch_application_from_command window=%p: %s",
 					    view->details->window, file_uri);
-			nautilus_launch_application_from_command (screen, NULL, file_uri, NULL, FALSE);
+			nautilus_launch_application_from_command (screen, NULL, file_uri, FALSE, NULL);
 			g_free (file_uri);
 		}
 		   
 	} else if (uri != NULL) {
-		if (view->details->selection_location == NULL ||
+		if (!open_in_same_slot ||
+		    view->details->selection_location == NULL ||
 		    strcmp (uri, view->details->selection_location) != 0) {
-			if (view->details->selection_location != NULL) {
-				g_free (view->details->selection_location);
+			if (open_in_same_slot) {
+				if (view->details->selection_location != NULL) {
+					g_free (view->details->selection_location);
+				}
+				view->details->selection_location = g_strdup (uri);
 			}
-			view->details->selection_location = g_strdup (uri);
+
 			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
 					    "tree view window_info_open_location window=%p: %s",
 					    view->details->window, uri);
 			location = g_file_new_for_uri (uri);
-			nautilus_window_info_open_location
-				(view->details->window, 
+			nautilus_window_slot_info_open_location
+				(slot,
 				 location,
-				 mode,
-				 0,
+				 NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+				 view->details->activation_flags,
 				 NULL);
 			g_object_unref (location);
 		}
@@ -445,7 +453,7 @@ selection_changed_timer_callback(FMTreeView *view)
 	if (view->details->activation_file == NULL) {
 		return FALSE;
 	}
-	view->details->activation_in_new_window = FALSE;
+	view->details->activation_flags = 0;
 		
 	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
@@ -668,8 +676,8 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 	gboolean can_delete_file;
 
 	if (event->button == 3) {
-		gboolean unmount_is_eject = FALSE;
 		gboolean show_unmount = FALSE;
+		gboolean show_eject = FALSE;
 		GMount *mount = NULL;
 		
 		if (!gtk_tree_view_get_path_at_pos (treeview, event->x, event->y,
@@ -721,22 +729,28 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		
 		mount = fm_tree_model_get_mount_for_root_node_file (view->details->child_model, view->details->popup_file);
 		if (mount) {
-			show_unmount = g_mount_can_unmount (mount) || g_mount_can_eject (mount);
 			/* TODO: show both unmount and eject if there are more than one volume for the drive */
-			unmount_is_eject = g_mount_can_eject (mount);
+			show_unmount = g_mount_can_unmount (mount);
+			show_eject = g_mount_can_eject (mount);
 		} 
 		
-		gtk_label_set_text (GTK_LABEL (GTK_BIN (GTK_MENU_ITEM (view->details->popup_unmount))->child),
-				    unmount_is_eject? _("E_ject"):_("_Unmount Volume"));
-		gtk_label_set_use_underline (GTK_LABEL (GTK_BIN (GTK_MENU_ITEM (view->details->popup_unmount))->child),
-				    TRUE);
 		if (show_unmount) {
-			gtk_widget_show (view->details->popup_unmount_separator);
 			gtk_widget_show (view->details->popup_unmount);
 		} else {
-			gtk_widget_hide (view->details->popup_unmount_separator);
 			gtk_widget_hide (view->details->popup_unmount);
-		}		
+		}
+
+		if (show_eject) {
+			gtk_widget_show (view->details->popup_eject);
+		} else {
+			gtk_widget_hide (view->details->popup_eject);
+		}
+
+		if (show_unmount || show_eject) {
+			gtk_widget_show (view->details->popup_unmount_separator);
+		} else {
+			gtk_widget_hide (view->details->popup_unmount_separator);
+		}
 
 		g_object_ref (view);
 		
@@ -752,6 +766,26 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 		
 		g_object_unref (view);
 
+		return TRUE;
+	} else if (event->button == 2 && event->type == GDK_BUTTON_PRESS) {
+		NautilusFile *file;
+
+		if (!gtk_tree_view_get_path_at_pos (treeview, event->x, event->y,
+						    &path, NULL, NULL, NULL)) {
+			return FALSE;
+		}
+
+		file = sort_model_path_to_file (view, path);
+		if (file) {
+			fm_tree_view_activate_file (view, file, 
+						    (event->state & GDK_CONTROL_MASK) != 0 ?
+						    NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW :
+						    NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+			nautilus_file_unref (file);
+		}
+
+		gtk_tree_path_free (path);
+
 		return TRUE;
 	}
 
@@ -761,14 +795,21 @@ button_pressed_callback (GtkTreeView *treeview, GdkEventButton *event,
 static void
 fm_tree_view_activate_file (FMTreeView *view, 
 			    NautilusFile *file,
-			    gboolean open_in_new_window)
+			    NautilusWindowOpenFlags flags)
 {
 	NautilusFileAttributes attributes;
 
 	cancel_activation (view);
 
+	if (view->details->activation_flags & NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB &&
+	    !eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		view->details->activation_flags &= ~NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+		view->details->activation_flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+	}
+
+
 	view->details->activation_file = nautilus_file_ref (file);
-	view->details->activation_in_new_window = open_in_new_window;
+	view->details->activation_flags = flags;
 		
 	attributes = NAUTILUS_FILE_ATTRIBUTE_INFO | NAUTILUS_FILE_ATTRIBUTE_LINK_INFO;
 	nautilus_file_call_when_ready (view->details->activation_file, attributes,
@@ -779,14 +820,21 @@ static void
 fm_tree_view_open_cb (GtkWidget *menu_item,
 		      FMTreeView *view)
 {
-	fm_tree_view_activate_file (view, view->details->popup_file, FALSE);
+	fm_tree_view_activate_file (view, view->details->popup_file, 0);
+}
+
+static void
+fm_tree_view_open_in_new_tab_cb (GtkWidget *menu_item,
+				    FMTreeView *view)
+{
+	fm_tree_view_activate_file (view, view->details->popup_file, NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
 }
 
 static void
 fm_tree_view_open_in_new_window_cb (GtkWidget *menu_item,
 				    FMTreeView *view)
 {
-	fm_tree_view_activate_file (view, view->details->popup_file, TRUE);
+	fm_tree_view_activate_file (view, view->details->popup_file, NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
 }
 
 static void
@@ -819,46 +867,74 @@ fm_tree_view_create_folder_cb (GtkWidget *menu_item,
 	g_free (parent_uri);
 }
 
+typedef struct {
+        char *file_uri;
+	gboolean cut;
+} ClipboardInfo;
+
 static void
 get_clipboard_callback (GtkClipboard	 *clipboard,
 			GtkSelectionData *selection_data,
-			guint		  info,
-			gpointer	  user_data_or_owner)
+			guint		  info_,
+			gpointer	  user_data)
 {
-	char *str = user_data_or_owner;
-	
-	gtk_selection_data_set (selection_data,
-				copied_files_atom,
-				8,
-				str,
-				strlen (str));
+	ClipboardInfo *info = user_data;
+
+
+        if (gtk_targets_include_uri (&selection_data->target, 1)) {
+                char *uris[2];
+
+                uris[0] = info->file_uri;
+                uris[1] = NULL;
+                gtk_selection_data_set_uris (selection_data, uris);
+        } else if (gtk_targets_include_text (&selection_data->target, 1)) {
+                GFile *file;
+                char *str;
+
+                file = g_file_new_for_uri (info->file_uri);
+                str = g_file_get_parse_name (file);
+                g_object_unref (file);
+
+                if (str) {
+                        gtk_selection_data_set_text (selection_data, str, strlen (str));
+                        g_free (str);
+                } else {
+                        gtk_selection_data_set_text (selection_data,
+                                                    info->file_uri, strlen (info->file_uri));
+                }
+        } else if (selection_data->target == copied_files_atom) {
+                char *str;
+
+                str = g_strdup_printf ("%s\n%s",
+                                       info->cut ? "cut" : "copy",
+                                       info->file_uri);
+                gtk_selection_data_set (selection_data, copied_files_atom,
+                                        8, str, strlen (str));
+                g_free (str);
+        }
 }
 
 static void
 clear_clipboard_callback (GtkClipboard *clipboard,
-			  gpointer	user_data_or_owner)
+			  gpointer	user_data)
 {
-	g_free (user_data_or_owner);
+	ClipboardInfo *info = user_data;
+
+        g_free (info->file_uri);
+	g_slice_free (ClipboardInfo, info);
 }
 
-static char *
-convert_file_to_string (NautilusFile *file,
-			gboolean cut)
+static ClipboardInfo *
+convert_file_to_uri (NautilusFile *file,
+		     gboolean cut)
 {
-	GString *uris;
-	char *uri, *result;
+        ClipboardInfo *info;
 
-	uris = g_string_new (cut ? "cut" : "copy");
-	
-	uri = nautilus_file_get_uri (file);
-	g_string_append_c (uris, '\n');
-	g_string_append (uris, uri);
-	g_free (uri);
+        info = g_slice_new (ClipboardInfo);
+        info->cut = cut;
+	info->file_uri = nautilus_file_get_uri (file);
 
-	result = uris->str;
-	g_string_free (uris, FALSE);
-
-	return result;
+        return info;
 }
 
 static void
@@ -866,14 +942,27 @@ copy_or_cut_files (FMTreeView *view,
 		   gboolean cut)
 {
 	char *status_string, *name;
-	char *clipboard_string;
-	
-	clipboard_string = convert_file_to_string (view->details->popup_file, cut);
+        ClipboardInfo *info;
+        GtkTargetList *target_list;
+        GtkTargetEntry *targets;
+        int n_targets;
+
+	info = convert_file_to_uri (view->details->popup_file, cut);
 	
+        target_list = gtk_target_list_new (NULL, 0);
+        gtk_target_list_add (target_list, copied_files_atom, 0, 0);
+        gtk_target_list_add_uri_targets (target_list, 0);
+        gtk_target_list_add_text_targets (target_list, 0);
+
+        targets = gtk_target_table_new_from_list (target_list, &n_targets);
+        gtk_target_list_unref (target_list);
+
 	gtk_clipboard_set_with_data (nautilus_clipboard_get (GTK_WIDGET (view->details->tree_widget)),
-				     clipboard_targets, G_N_ELEMENTS (clipboard_targets),
+				     targets, n_targets,
 				     get_clipboard_callback, clear_clipboard_callback,
-				     clipboard_string);
+				     info);
+        gtk_target_table_free (targets, n_targets);
+
 	nautilus_clipboard_monitor_emit_changed ();
 	
 	name = nautilus_file_get_display_name (view->details->popup_file);
@@ -888,8 +977,8 @@ copy_or_cut_files (FMTreeView *view,
 	}
 	g_free (name);
 	
-	nautilus_window_info_set_status (view->details->window,
-					 status_string);
+	nautilus_window_info_push_status (view->details->window,
+					  status_string);
 	g_free (status_string);
 }
 
@@ -920,8 +1009,8 @@ paste_clipboard_data (FMTreeView *view,
 									 copied_files_atom);
 
 	if (item_uris == NULL|| destination_uri == NULL) {
-		nautilus_window_info_set_status (view->details->window,
-						 _("There is nothing on the clipboard to paste."));
+		nautilus_window_info_push_status (view->details->window,
+						  _("There is nothing on the clipboard to paste."));
 	} else {
 		nautilus_file_operations_copy_move
 			(item_uris, NULL, destination_uri,
@@ -1043,8 +1132,26 @@ fm_tree_view_unmount_cb (GtkWidget *menu_item,
 	
 	if (mount != NULL) {
 		nautilus_file_operations_unmount_mount (fm_tree_view_get_containing_window (view),
-							mount,
-							g_mount_can_eject (mount), TRUE);
+							mount, FALSE, TRUE);
+	}
+}
+
+static void
+fm_tree_view_eject_cb (GtkWidget *menu_item,
+		       FMTreeView *view)
+{
+	NautilusFile *file = view->details->popup_file;
+	GMount *mount;
+	
+	if (file == NULL) {
+		return;
+	}
+
+	mount = fm_tree_model_get_mount_for_root_node_file (view->details->child_model, file);
+	
+	if (mount != NULL) {
+		nautilus_file_operations_unmount_mount (fm_tree_view_get_containing_window (view),
+							mount, TRUE, TRUE);
 	}
 }
 
@@ -1074,8 +1181,17 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_open = menu_item;
 	
+	/* add the "open in new tab" menu item */
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Open in New _Tab"));
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (fm_tree_view_open_in_new_tab_cb),
+			  view);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+	view->details->popup_open_in_new_window = menu_item;
+	
 	/* add the "open in new window" menu item */
-	menu_item = gtk_image_menu_item_new_with_label (_("Open in New Window"));
+	menu_item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_open_in_new_window_cb),
 			  view);
@@ -1156,7 +1272,7 @@ create_popup_menu (FMTreeView *view)
 	eel_gtk_menu_append_separator (GTK_MENU (popup));
 
 	/* add the "Unmount" menu item */
-	menu_item = gtk_image_menu_item_new_with_label ("eject label");
+	menu_item = gtk_image_menu_item_new_with_mnemonic ("_Unmount");
 	g_signal_connect (menu_item, "activate",
 			  G_CALLBACK (fm_tree_view_unmount_cb),
 			  view);
@@ -1164,6 +1280,15 @@ create_popup_menu (FMTreeView *view)
 	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
 	view->details->popup_unmount = menu_item;
 
+	/* add the "Eject" menu item */
+	menu_item = gtk_image_menu_item_new_with_mnemonic ("E_ject");
+	g_signal_connect (menu_item, "activate",
+			  G_CALLBACK (fm_tree_view_eject_cb),
+			  view);
+	gtk_widget_show (menu_item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (popup), menu_item);
+	view->details->popup_eject = menu_item;
+
 	/* add the unmount separator menu item */
 	view->details->popup_unmount_separator =
 		GTK_WIDGET (eel_gtk_menu_append_separator (GTK_MENU (popup)));
@@ -1190,6 +1315,7 @@ create_tree (FMTreeView *view)
 	GList *mounts, *l;
 	char *location;
 	GIcon *icon;
+	NautilusWindowSlotInfo *slot;
 	
 	view->details->child_model = fm_tree_model_new ();
 	view->details->sort_model = GTK_TREE_MODEL_SORT
@@ -1290,7 +1416,8 @@ create_tree (FMTreeView *view)
 			  "button_press_event", G_CALLBACK (button_pressed_callback),
 			  view);
 
-	location = nautilus_window_info_get_current_location (view->details->window);
+	slot = nautilus_window_info_get_active_slot (view->details->window);
+	location = nautilus_window_slot_info_get_current_location (slot);
 	schedule_select_and_show_location (view, location);
 	g_free (location);
 }
@@ -1315,9 +1442,6 @@ update_filtering_from_preferences (FMTreeView *view)
 			(view->details->child_model,
 			 mode == NAUTILUS_WINDOW_SHOW_HIDDEN_FILES_ENABLE);
 	}
-	fm_tree_model_set_show_backup_files
-		(view->details->child_model,
-		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_SHOW_BACKUP_FILES));
 	fm_tree_model_set_show_only_directories
 		(view->details->child_model,
 		 eel_preferences_get_boolean (NAUTILUS_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES));
@@ -1347,8 +1471,11 @@ filtering_changed_callback (gpointer callback_data)
 static void
 loading_uri_callback (NautilusWindowInfo *window,
 		      char *location,
-		      FMTreeView *view)
+		      gpointer callback_data)
 {
+	FMTreeView *view;
+
+	view = FM_TREE_VIEW (callback_data);
 	schedule_select_and_show_location (view, location);
 }
 
@@ -1508,12 +1635,15 @@ fm_tree_view_set_parent_window (FMTreeView *sidebar,
 				NautilusWindowInfo *window)
 {
 	char *location;
+	NautilusWindowSlotInfo *slot;
 	
 	sidebar->details->window = window;
 
+	slot = nautilus_window_info_get_active_slot (window);
+
 	g_signal_connect_object (window, "loading_uri",
 				 G_CALLBACK (loading_uri_callback), sidebar, 0);
-	location = nautilus_window_info_get_current_location (window);
+	location = nautilus_window_slot_info_get_current_location (slot);
 	loading_uri_callback (window, location, sidebar);
 	g_free (location);
 
