@@ -31,12 +31,7 @@
 #include <eel/eel-gtk-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-preferences.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkenums.h>
-#include <gtk/gtkliststore.h>
-#include <gtk/gtktreemodel.h>
-#include <gtk/gtktreeselection.h>
+#include <gtk/gtk.h>
 #include <libgnome/gnome-macros.h>
 #include <glib/gi18n.h>
 #include <libnautilus-private/nautilus-bookmark.h>
@@ -44,6 +39,8 @@
 #include <libnautilus-private/nautilus-sidebar-provider.h>
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-signaller.h>
+#include <libnautilus-private/nautilus-window-info.h>
+#include <libnautilus-private/nautilus-window-slot-info.h>
 
 #include "nautilus-history-sidebar.h"
 
@@ -136,19 +133,17 @@ history_changed_callback (GObject *signaller,
 }
 
 static void
-row_activated_callback (GtkTreeView *tree_view,
-			GtkTreePath *path,
-			GtkTreeViewColumn *column,
-			gpointer user_data)
+open_selected_item (NautilusHistorySidebar *sidebar, 
+		    GtkTreePath *path,
+		    NautilusWindowOpenFlags flags)
 {
-	NautilusHistorySidebar *sidebar;
+	NautilusWindowSlotInfo *slot;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
 	NautilusBookmark *bookmark;
-	char *uri;
+	GFile *location;
 	
-	sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
-	model = gtk_tree_view_get_model (tree_view);
+	model = gtk_tree_view_get_model (sidebar->tree_view);
 	
 	if (!gtk_tree_model_get_iter (model, &iter, path)) {
 		return;
@@ -158,11 +153,53 @@ row_activated_callback (GtkTreeView *tree_view,
 		(model, &iter, HISTORY_SIDEBAR_COLUMN_BOOKMARK, &bookmark, -1);
 	
 	/* Navigate to the clicked location. */
-	uri = nautilus_bookmark_get_uri (NAUTILUS_BOOKMARK (bookmark));
-	nautilus_window_info_open_location
-		(sidebar->window, 
-		 uri, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0, NULL);
-	g_free (uri);
+	location = nautilus_bookmark_get_location (NAUTILUS_BOOKMARK (bookmark));
+	slot = nautilus_window_info_get_active_slot (sidebar->window);
+	nautilus_window_slot_info_open_location
+		(slot,
+		 location, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 
+		 flags, NULL);
+	g_object_unref (location);
+}
+
+static void
+row_activated_callback (GtkTreeView *tree_view,
+			GtkTreePath *path,
+			GtkTreeViewColumn *column,
+			gpointer user_data)
+{
+	NautilusHistorySidebar *sidebar;
+	
+	sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
+	g_assert (sidebar->tree_view == tree_view);
+
+	open_selected_item (sidebar, path, 0);
+}
+
+static gboolean
+button_press_event_callback (GtkWidget *widget,
+			     GdkEventButton *event,
+			     gpointer user_data)
+{
+	if (event->button == 2 && event->type == GDK_BUTTON_PRESS) {
+		/* Open new tab on middle click. */
+		NautilusHistorySidebar *sidebar;
+		GtkTreePath *path;
+
+		sidebar = NAUTILUS_HISTORY_SIDEBAR (user_data);
+		g_assert (sidebar->tree_view == GTK_TREE_VIEW (widget));
+
+		if (gtk_tree_view_get_path_at_pos (sidebar->tree_view,
+						   event->x, event->y,
+						   &path, NULL, NULL, NULL)) {
+			open_selected_item (sidebar, 
+					    path,
+					    NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB);
+			gtk_tree_path_free (path);
+		}
+	}
+
+	return FALSE;
 }
 
 static void
@@ -245,6 +282,9 @@ nautilus_history_sidebar_init (NautilusHistorySidebar *sidebar)
 				 "history_list_changed",
 				 G_CALLBACK (history_changed_callback), sidebar, 0);
 
+	g_signal_connect (tree_view, "button-press-event", 
+			  G_CALLBACK (button_press_event_callback), sidebar);
+
 	eel_preferences_add_callback (NAUTILUS_PREFERENCES_CLICK_POLICY,
 				      click_policy_changed_callback,
 				      sidebar);
